#!/usr/bin/perl -w
#
# fcs-client.pl
#

use strict;
use LWP::UserAgent;
use HTTP::Request::Common;
use File::Basename;
use Getopt::Mixed "nextOption";

# default configuration
chomp(my $hostname = `hostname`);
my %cfg = (
		version		=> '0.1',
		url 		=> 'http://osiris.linuxminded.xs4all.nl/folding/index.pl',
		unitinfo_file	=> '/home/bas/software/folding/unitinfo.txt',
		config_file	=> '/home/bas/software/folding/client.cfg',
		client_name	=> $hostname,
		useragent	=> 'fci-client/0.1',
		verbose		=> 0
	);

# get command line parameters
Getopt::Mixed::init("url=s client=s config=s unitinfo=s useragent=s verbose help");
while (my ($option, $value) = nextOption()) {
	if($option eq 'help') {
		print_usage();
		Getopt::Mixed::cleanup();
		exit;
	}
	
	$cfg{url} 	    = $value if($option eq 'url');
	$cfg{client_name}   = $value if($option eq 'client');
	$cfg{config_file}   = $value if($option eq 'config');
	$cfg{unitinfo_file} = $value if($option eq 'unitinfo');
	$cfg{useragent}	    = $value if($option eq 'useragent');
	$cfg{verbose} 	    = 1      if($option eq 'verbose');
}
Getopt::Mixed::cleanup();

# check file existance & other parameters
if(!-r $cfg{config_file}) {
	print "Error: Cannot read client configuration file! ($cfg{config_file})\n";
	exit;
}
elsif(!-r $cfg{unitinfo_file}) {
	print "Error: Cannot read unitinfo file!\n";
	exit;
}
elsif(!$cfg{client_name} || $cfg{client_name} eq '' || $cfg{client_name} =~ /^\s+$/) {
	print "Error: Invalid client name! ($cfg{client_name})\n";
	exit;
}
elsif(!$cfg{useragent} || $cfg{useragent} eq '' || $cfg{useragent} =~ /^\s+$/) {
	print "Error: Invalid User-Agent! ($cfg{useragent})\n";
	exit;
}
elsif($cfg{url} !~ /^http:\/\// || &invalid_url($cfg{url})) {
	print "Error: Invalid URL or HTTP error! ($cfg{url})\n";
	exit;
}

# print configuration if verbose output is requested
if($cfg{verbose}) {
	print basename($0)." started\n\n";
	print "going to submit the following data to $cfg{url}:\n";
	print "client name   : $cfg{client_name}\n";
	print "config file   : $cfg{config_file}\n";
	print "unitinfo file : $cfg{unitinfo_file}\n\n";
}

# post file data to the server
my $ua  = LWP::UserAgent->new;
   $ua->agent($cfg{useragent});
my $req = POST $cfg{url},
	  Content_Type => 'form-data',
	  Content      => [
		  	   submit        => 1,
			   action        => 'upload',
			   client_name   => $cfg{client_name},
			   unitinfo_file => [ $cfg{unitinfo_file} ],
			   config_file   => [ $cfg{config_file} ]
			  ];

my $res = $ua->request($req);

if ($res->is_success()) {
	if($cfg{verbose} == 1) {
		print "Successfully posted data to the server.\n";
		print $res->content."\n";
	}
} 
else {
	print "Error uploading unitinfo file ($cfg{unitinfo_file}) and config file ($cfg{config_file}) ";
	print "to url ($cfg{url}) for client ($cfg{client_name}) with useragent ($cfg{useragent})!\n";
}

################[ Subroutines ]################

sub invalid_url {
	my $url = $_[0];

	my $ua  = LWP::UserAgent->new();
	   $ua->agent($cfg{useragent});
	my $req = HTTP::Request->new(HEAD => $url);

	my $res = $ua->request($req);
	if($res->is_success()) {
		return 0;
	}
	elsif($res->is_error()) {
		return 1;
	}
	else {
		return 1;
	}
}

sub print_usage {
	my $verbose = $cfg{verbose} ? 'on' : 'off';

	print "Usage: ".basename($0)." [OPTIONS]\n\n";
	print "Options:\n";
	print "--url <url>        URL to POST the data to        (default: $cfg{url})\n";
	print "--client <name>    Client name                    (default: $cfg{client_name})\n";
	print "--config <file>    Path to the client.cfg file    (default: $cfg{config_file})\n";
	print "--unitinfo <file>  Path to the unitinfo.txt file  (default: $cfg{unitinfo_file})\n";
	print "--useragent <name> User-Agent string		 (default: $cfg{useragent})\n";
	print "--verbose          Verbose (debug) output         (default: $verbose)\n";
	print "--help             Print this information\n";
}
